from .commons import run_client, get_basic_arg_parser

def _runner(client, path, args):

    services = client.discover_services(uri=path)

    for s in services:
        if args.as_xml:
            print s.raw.to_xml(pretty_print=True)
        else:
            print s.raw.to_text()


def discover_services():
    run_client(get_basic_arg_parser(), _runner)





import os
import sys
import re
import logging
import hashlib
import dateutil.parser

from .commons import run_client, get_basic_arg_parser

log = logging.getLogger(__name__)


def extend_arguments(parser):
    parser.add_argument("-c", "--collection", dest="collection", help="collection to poll", required=True)
    parser.add_argument("--dest-dir", dest="dest_dir", help="directory to save polled content")

    parser.add_argument("-l", "--limit", dest="limit", type=int, default=sys.maxint,
            help="limit the number of content blocks returned")

    parser.add_argument("-r", "--raw", dest="as_raw", action='store_true',
            help="output raw TAXII messages "\
                 "(use in combination with -x to see them as XML)")

    parser.add_argument("--begin", dest="begin", help="exclusive beginning of time window as ISO8601 formatted date")
    parser.add_argument("--end", dest="end", help="inclusive ending of time window as ISO8601 formatted date")

    parser.add_argument("-s", "--subscription", dest="subscription_id", help="ID of an existing subscription")

    return parser


def generate_filename(collection, content_block):

    collection_name = re.sub(r"[^\w]+", "-", collection) if collection else ""

    md5 = hashlib.md5()
    md5.update(content_block.to_xml())

    filename = '%s_%s' % (collection_name, md5.hexdigest())

    return filename


def save_to_dir(dest_dir, collection, content_block, as_raw):

    filename = generate_filename(collection, content_block)
    path = os.path.abspath(os.path.join(dest_dir, filename))

    with open(path, 'w') as f:
        if as_raw:
            content = content_block.to_xml(pretty_print=True)
        else:
            content = content_block.content

        f.write(content)

    log.info("Content block saved to %s", path)



def _runner(client, path, args):

    if args.limit == 0:
        return

    begin = dateutil.parser.parse(args.begin) if args.begin else None
    end = dateutil.parser.parse(args.end) if args.end else None

    blocks = client.poll(args.collection, begin_date=begin, end_date=end,
            subscription_id=args.subscription_id, uri=path)

    counter = 0

    for counter, block in enumerate(blocks, 1):
        if args.dest_dir:
            dest_dir = os.path.abspath(args.dest_dir)
            save_to_dir(dest_dir, args.collection, block, args.as_raw)
        else:
            if args.as_raw:
                if args.as_xml:
                    print block.to_xml()
                else:
                    print block.to_text()
            else:
                print block.content

        if counter >= args.limit:
            break

    log.info("%d blocks polled", counter)


def poll_content():
    run_client(extend_arguments(get_basic_arg_parser()), _runner)





from libtaxii.constants import SVC_COLLECTION_MANAGEMENT, SVC_FEED_MANAGEMENT

from .commons import run_client, get_basic_arg_parser

def _get_collections(client, path):

    if hasattr(client, 'get_collections'):
        return client.get_collections(uri=path)
    elif hasattr(client, 'get_feeds'):
        return client.get_feeds(uri=path)


def _runner(client, path, args):

    collections = []

    if path:
        collections.extend(_get_collections(client, path))
    else:
        for service in client.get_services(service_types=[SVC_COLLECTION_MANAGEMENT, SVC_FEED_MANAGEMENT]):
            _collections = _get_collections(client, service.service_address)
            collections.extend(_collections)

    for c in collections:
        if args.as_xml:
            print c.raw.to_xml(pretty_print=True)
        else:
            print c.raw.to_text()


def fetch_collections():
    run_client(get_basic_arg_parser(), _runner)





import logging
import urlparse
import urllib2

from libtaxii.clients import HttpClient
from libtaxii.common import generate_message_id
from libtaxii import constants as const
from libtaxii import get_message_from_http_response

from .converters import to_detailed_service_instance_entity
from .utils import configure_client_auth
from .exceptions import (
    NoURIProvidedError, UnsuccessfulStatusError, ServiceNotFoundError,
    AmbiguousServicesError, ClientException, HTTPError
)


class AbstractClient(object):
    '''Abstract client class.

    This class can not be used directly, use :py:meth:`cabby.create_client`
    to create client instances.
    '''

    NO_PROXY = HttpClient.NO_PROXY
    PROXY_TYPE_CHOICES = [HttpClient.PROXY_HTTP, HttpClient.PROXY_HTTPS]
    SUPPORTED_SCHEMES = ['http', 'https']

    def __init__(self, host=None, discovery_path=None, port=None,
            use_https=False, headers=None):

        self.host = host
        self.port = port or (443 if use_https else 80)
        self.use_https = use_https

        self.discovery_path = discovery_path
        self.services = None

        self.auth = None
        self.proxy_details = None

        self.headers = headers

        self.log = logging.getLogger("%s.%s" % (self.__module__,
            self.__class__.__name__))

    def set_auth(self, cert_file=None, key_file=None, username=None,
            password=None):
        '''Set authentication credentials.

        Authentication types can be combined. It is possible to use
        only SSL authentication, only basic authentication, or 
        basic authentication over SSL.

        :param str cert_file: a path to SSL certificate file
        :param str key_file: a path to SSL key file
        :param str username: basic authentication username
        :param str password: basic authentication password
        '''

        self.auth = {
            'cert_file' : cert_file,
            'key_file' : key_file,
            'username' : username,
            'password' : password
        }

    def set_proxy(self, proxy_url, proxy_type=None):
        '''Set proxy properties.

        :param str proxy_url: proxy address formated as an URL or
                              :attr:`NO_PROXY` to force client not
                              to use proxy.
        :param str proxy_type: one of the values
                              from :attr:`PROXY_TYPE_CHOICES`
        '''

        if not proxy_url:
            return ValueError('proxy_url can not be None')

        if (proxy_url != NO_PROXY and not proxy_type) or \
                (proxy_type and proxy_type not in PROXY_TYPE_CHOICES):
            types = ", ".join(PROXY_TYPE_CHOICES)
            return ValueError('proxy_type needs to to be one of: %s' % types)

        self.proxy_details = {
            'proxy_type' : proxy_type,
            'proxy_string' : proxy_url
        }

    @staticmethod
    def _create_client(auth=None, use_https=False, proxy_details=None):

        client = HttpClient(use_https=use_https)
        client = configure_client_auth(client, **(auth or {}))

        if proxy_details:
            client.set_proxy(**proxy_details)

        return client

    def _execute_request(self, request, uri=None, service_type=None):
        '''Execute generic TAXII request.

        A service is defined by ``uri`` parameter or is chosen from pre-cached
        services by ``service_type``.
        '''
        
        if not uri and not service_type:
            raise NoURIProvidedError('URI or service_type needs to be provided')
        elif not uri:
            service = self._get_service(service_type)
            uri = service.address

        parsed = urlparse.urlparse(uri)
        if not parsed.scheme:
            # faking schema because otherwise urlparse gets confused
            parsed = urlparse.urlparse("http://" + uri)
        elif parsed.scheme not in self.SUPPORTED_SCHEMES:
            raise ValueError('Scheme "%s" is not supported. Use one of: %s' % \
                    (parsed.scheme, ', '.join(self.SUPPORTED_SCHEMES)))

        use_https = self.use_https or (parsed.scheme == 'https')

        host = parsed.hostname or self.host
        port = parsed.port or (443 if use_https else self.port)
        path = parsed.path

        full_path = "http%(https)s://%(host)s:%(port)s%(path)s" % dict(
                https=('s' if use_https else ''), host=host, port=port,
                path=path)

        if not host:
            raise ValueError('Host name is not provided: %s' % full_path)

        client = AbstractClient._create_client(
                auth=self.auth, use_https=use_https)

        request_body = request.to_xml(pretty_print=True)

        full_path = "http%(https)s://%(host)s:%(port)s%(path)s" % dict(
                https=('s' if use_https else ''), host=host, port=port,
                path=path)
        self.log.info("Sending %s to %s", request.message_type, full_path)
        self.log.debug("Request:\n%s", request_body)

        response_raw = client.call_taxii_service2(
            host, path, self.taxii_version, request_body, port=port,
            headers=self.headers)

        # https://github.com/TAXIIProject/libtaxii/issues/181
        if isinstance(response_raw, urllib2.URLError):
            error = response_raw
            self.log.debug("%s: %s", error, error.read())
            raise HTTPError(error)

        response = get_message_from_http_response(
            response_raw, in_response_to='0')

        self.log.info("Response received for %s from %s",
                request.message_type, full_path)

        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Response:\n%s", response.to_xml(pretty_print=True))

        if hasattr(response, 'status_type'):
            if response.status_type != const.ST_SUCCESS:
                raise UnsuccessfulStatusError(response)
        else:
            return response

    def _generate_id(self):
        return generate_message_id(version=self.services_version)

    def _get_service(self, service_type):

        candidates = self.get_services(service_type=service_type)

        if not candidates:
            raise ServiceNotFoundError(
                "Service with type '%s' is not advertised" % service_type)
        elif len(candidates) > 1:
            raise AmbiguousServicesError(
                "%d services found with type '%s'. Specify the exact URI" % (
                    len(candidates), service_type))

        return candidates[0]

    def get_services(self, service_type=None, service_types=None):
        '''Get services advertised by TAXII server.

        This method will try to do automatic discovery by calling
        :py:meth:`discover_services`.

        :param str service_type: filter services by specific type. Accepted
                                 values are listed in
                                 :py:data:`cabby.entities.SERVICE_TYPES`
        :param str service_types: filter services by multiple types. Accepted
                                 values are listed in
                                 :py:data:`cabby.entities.SERVICE_TYPES`

        :return: list of service instances
        :rtype: list of :py:class:`cabby.entities.DetailedServiceInstance`
                (or :py:class:`cabby.entities.InboxDetailedService`)
        
        :raises ValueError:
                if URI provided is invalid or schema is not supported
        :raises `cabby.exceptions.HTTPError`:
                if HTTP error happened
        :raises `cabby.exceptions.UnsuccessfulStatusError`:
                if Status Message received and status_type is not `SUCCESS`
        :raises `cabby.exceptions.ServiceNotFoundError`:
                if no service found
        :raises `cabby.exceptions.AmbiguousServicesError`:
                more than one service with type specified
        :raises `cabby.exceptions.NoURIProvidedError`:
                no URI provided and client can't discover services
        '''
        if not self.services:
            try:
                services = self.discover_services()
            except ClientException, e:
                self.log.error('Can not autodiscover advertised services')
                raise e
        else:
            services = self.services

        if not service_type and not service_types:
            return services

        if service_type:
            filter_func = lambda s: s.type == service_type
        elif service_types:
            filter_func = lambda s: s.type in service_types

        return filter(filter_func, services)

    def discover_services(self, uri=None, cache=True):
        '''Discover services advertised by TAXII server.

        This method will send discovery request to a service, defined by ``uri``
        or constructor's connection parameters.

        :param str uri: URI path to a specific TAXII service
        :param bool cache: if discovered services should be cached

        :return: list of TAXII services
        :rtype: list of :py:class:`cabby.entities.DetailedServiceInstance`
                (or :py:class:`cabby.entities.InboxDetailedService`)

        :raises ValueError:
                if URI provided is invalid or schema is not supported
        :raises `cabby.exceptions.HTTPError`:
                if HTTP error happened
        :raises `cabby.exceptions.UnsuccessfulStatusError`:
                if Status Message received and status_type is not `SUCCESS`
        :raises `cabby.exceptions.ServiceNotFoundError`:
                if no service found
        :raises `cabby.exceptions.AmbiguousServicesError`:
                more than one service with type specified
        :raises `cabby.exceptions.NoURIProvidedError`:
                no URI provided and client can't discover services
        '''

        uri = uri or self.discovery_path

        if not uri:
            raise NoURIProvidedError('Discovery service URI is not specified')

        response = self._discovery_request(uri)

        services = map(to_detailed_service_instance_entity,
                response.service_instances)

        self.log.info("%d services discovered", len(services))

        if cache:
            self.services = services

        return services




import urlparse
import logging
import sys
import argparse
from colorlog import ColoredFormatter

from libtaxii.clients import HttpClient

from ..abstract import AbstractClient
from ..exceptions import HTTPError
from .. import create_client

log = logging.getLogger(__name__)

TAXII_10 = '1.0'
TAXII_11 = '1.1'
DEFAULT_VERSION = TAXII_11

VERSION_CHOICES = [TAXII_10, TAXII_11]

def get_basic_arg_parser():

    parser = argparse.ArgumentParser(
        description = "TAXII client",
        formatter_class = argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument("--host", dest="host", help="host where the TAXII Service is hosted")
    parser.add_argument("--port", dest="port", type=int, help="port where the TAXII Service is hosted")
    parser.add_argument("--discovery", dest="discovery", help="path to a TAXII Discovery service")
    parser.add_argument("--path", dest="uri", help="absolute path (as URL) or relative path to a specific TAXII Service")

    parser.add_argument("--https", dest="https", action='store_true', help="if the client should use HTTPS")

    parser.add_argument("--cert", dest="cert", help="certificate file")
    parser.add_argument("--key", dest="key", help="private key file")
    parser.add_argument("--username", dest="username", help="username for authentication")
    parser.add_argument("--password", dest="password", help="password for authentication")

    parser.add_argument("--proxy-url", dest="proxy_url", help="proxy address formatted as URL. Can be set to 'noproxy' to force library to not use any proxy")
    parser.add_argument("--proxy-type", dest="proxy_type", choices=AbstractClient.PROXY_TYPE_CHOICES, help="proxy type")

    parser.add_argument("--header", dest="headers", action='append', help="headers to send with the requests, as header:value pairs")

    parser.add_argument("-v", "--verbose", dest="verbose", action='store_true', help="verbose mode")
    parser.add_argument("-x", "--as-xml", dest="as_xml", action='store_true', help="output raw XML")

    parser.add_argument("-t", "--taxii-version", dest="version", default=DEFAULT_VERSION, choices=VERSION_CHOICES, help="TAXII version to use")

    return parser



def is_args_valid(args):

    if not args.uri and not args.discovery:
        log.error("Either 'discovery' or 'path' needs to be specified")
        return False

    return True
        

def prepare_headers(raw_headers):
    headers = {}

    for line in raw_headers:
        header, value = line.split(":", 1)
        headers[header] = value.strip('"\'')

    return headers


def run_client(parser, run_func):

    args = parser.parse_args()

    level = logging.DEBUG if args.verbose else logging.INFO
    configure_color_logging(level=level)

    if not is_args_valid(args):
        sys.exit(1)

    headers = prepare_headers(args.headers) if args.headers else None

    client = create_client(host=args.host, discovery_path=args.discovery, port=args.port,
            use_https=args.https, version=args.version, headers=headers)

    client.set_auth(
        cert_file = args.cert,
        key_file = args.key,
        username = args.username,
        password = args.password
    )

    try:
        run_func(client, args.uri, args)
    except Exception, e:
        log.error(e, exc_info=args.verbose)


def configure_color_logging(level, logger_name=None):

    logger = logging.getLogger(logger_name)
    logger.setLevel(level)

    if level == logging.DEBUG:
        format_string = "%(asctime)s %(name)s %(log_color)s%(levelname)s:%(reset)s %(white)s%(message)s"
    else:
        format_string = "%(asctime)s %(log_color)s%(levelname)s:%(reset)s %(white)s%(message)s"

    formatter = ColoredFormatter(
        format_string,
        datefmt = None,
        reset = True,
        log_colors = {
            'DEBUG': 'cyan',
            'INFO': 'green',
            'WARNING': 'yellow',
            'ERROR': 'red',
            'CRITICAL': 'red',
        }
    )

    handlers = logging.StreamHandler(sys.stderr)
    handlers.setFormatter(formatter)
    logger.addHandler(handlers)




import pytz
from datetime import datetime
from libtaxii.clients import HttpClient
import libtaxii.messages_11 as tm11

from .entities import ContentBinding

def get_utc_now():
    return datetime.utcnow().replace(tzinfo=pytz.UTC)


def configure_client_auth(tclient, cert_file=None, key_file=None,
        username=None, password=None):

    tls_auth = (cert_file and key_file)
    basic_auth = (username and password)

    if tls_auth and basic_auth:
        tclient.set_auth_type(HttpClient.AUTH_CERT_BASIC)
        tclient.set_auth_credentials(dict(
            key_file = key_file, 
            cert_file = cert_file,
            username = username,
            password = password
        ))
    elif tls_auth:
        tclient.set_auth_type(HttpClient.AUTH_CERT)
        tclient.set_auth_credentials(dict(
            key_file = key_file, 
            cert_file = cert_file,
        ))
    elif basic_auth:
        tclient.set_auth_type(HttpClient.AUTH_BASIC)
        tclient.set_auth_credentials(dict(
            username = username,
            password = password
        ))

    return tclient


def pack_content_binding(content_binding, version):
    if isinstance(content_binding, ContentBinding):
        if version == 11:
            binding = tm11.ContentBinding(binding_id=content_binding.id,
                subtype_ids=content_binding.subtypes)
        else:
            binding = content_binding.id
    else:
        if version == 11:
            binding = tm11.ContentBinding(binding_id=content_binding)
        else:
            binding = content_binding
    return binding


def pack_content_bindings(content_bindings, version):

    if not content_bindings:
        return None

    bindings = []

    for b in content_bindings:
        bindings.append(pack_content_binding(b, version))

    return bindings




from os.path import join, dirname
from setuptools import setup, find_packages

__version__ = None
execfile('cabby/_version.py')

CURRENT_DIR = dirname(__file__)

def get_file_contents(filename):
    with open(join(CURRENT_DIR, filename)) as fp:
        return fp.read()

setup(
    name = "cabby",
    description = "Python library for interacting with TAXII servers",
    long_description = get_file_contents('README.rst'),
    url = "https://github.com/Intelworks/cabby/",
    author = "Intelworks",
    author_email = "cabby@intelworks.com",
    version = __version__,
    license = "BSD License",
    packages = find_packages(exclude=['tests']),
    entry_points = {
        'console_scripts' : [
            'taxii-poll = cabby.cli:poll_content',
            'taxii-push = cabby.cli:push_content',
            'taxii-discovery = cabby.cli:discover_services',
            'taxii-collections = cabby.cli:fetch_collections',
            'taxii-subscription = cabby.cli:manage_subscription',
        ]
    },
    install_requires = [
        'pytz==2014.10',
        'colorlog',
        'libtaxii>=1.1.106'
    ],
    classifiers = [
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 2 :: Only",
        "Topic :: Internet",
        "Topic :: Security",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ]
)





class ClientException(Exception):
    pass

class HTTPError(ClientException):
    pass

class UnsuccessfulStatusError(ClientException):

    def __init__(self, taxii_status, *args, **kwargs):
        super(UnsuccessfulStatusError, self).__init__(_status_to_message(taxii_status), *args, **kwargs)

        self.status = taxii_status.status_type
        self.text = taxii_status.to_text()

        self.raw = taxii_status


class AmbiguousServicesError(ClientException):
    pass


class ServiceNotFoundError(ClientException):
    pass


class NoURIProvidedError(ValueError):
    pass


def _status_to_message(status):
    l = [status.status_type]

    if status.status_detail:
        l.append(_dict_to_pairs(status.status_detail))

    if status.extended_headers:
        l.append(_dict_to_pairs(status.extended_headers))
    
    if status.message:
        l.append(status.message)

    return "; ".join(l)


def _dict_to_pairs(d):
    pairs = []
    for k, v in d.items():
        pairs.append('%s=%s' % (k, v))
    return ", ".join(pairs)



import logging

from libtaxii.constants import CB_STIX_XML_111

from cabby.entities import ContentBinding

from .commons import run_client, get_basic_arg_parser

log = logging.getLogger(__name__)

DEFAULT_BINDING = CB_STIX_XML_111

def extend_arguments(parser):
    parser.add_argument("-f", "--content-file", dest="content_file", help="file with the content to send", required=True)
    parser.add_argument("--binding", dest="binding", default=DEFAULT_BINDING, help="content binding")
    parser.add_argument("--subtype", dest="subtypes", action='append', help="the subtype of the content binding")
    parser.add_argument("--dest", dest="collections", action='append', help="names of the destination collections")

    return parser


def _runner(client, path, args):

    with open(args.content_file, 'r') as f:
        content = f.read()

    if args.binding and args.subtypes:
        binding = ContentBinding(args.binding, subtypes=args.subtypes)
    elif args.binding:
        binding = args.binding
    else:
        binding = None

    client.push(content, binding, collections_names=args.collections, uri=path)

    log.info("Content pushed")


def push_content():
    run_client(extend_arguments(get_basic_arg_parser()), _runner)

